<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<xsl:stylesheet xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:saxon="http://saxon.sf.net/"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:schold="http://www.ascc.net/xml/schematron"
                xmlns:iso="http://purl.oclc.org/dsdl/schematron"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                xmlns:ucm="http://www.universalchemicalmarkup.org"
                xmlns:bibtexml="http://bibtexml.sf.net/"
                xmlns:unitsml="urn:oasis:names:tc:unitsml:schema:xsd:UnitsMLSchema-1.0"
                xmlns:lf="LocalFunctions"
                version="2.0"><!--Implementers: please note that overriding process-prolog or process-root is 
    the preferred method for meta-stylesheets to use where possible. -->
   <xsl:param name="archiveDirParameter"/>
   <xsl:param name="archiveNameParameter"/>
   <xsl:param name="fileNameParameter"/>
   <xsl:param name="fileDirParameter"/>
   <xsl:variable name="document-uri">
      <xsl:value-of select="document-uri(/)"/>
   </xsl:variable>

   <!--PHASES-->


   <!--PROLOG-->


   <!--XSD TYPES FOR XSLT2-->


   <!--KEYS AND FUNCTIONS-->
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:CHECK-REFS"
                 as="xs:boolean">

      <xsl:param name="REFS-ATTRIBUTE-NAME" as="xs:string"/>
      <xsl:param name="ELEMENT-WITH-REFS" as="node()"/>
      <xsl:param name="RELEVANT-ELEMENTS-WITH-ID" as="node()*"/>

      <xsl:variable name="RELEVANT-ID-LIST">
        <xsl:for-each select="$RELEVANT-ELEMENTS-WITH-ID">
            <xsl:value-of select="./@id"/>
            <xsl:if test="position() != last()">
                <xsl:text> </xsl:text>
            </xsl:if>
        </xsl:for-each>
      </xsl:variable>

      <xsl:variable name="CHECK-RESULT">
        <xsl:for-each select="tokenize($ELEMENT-WITH-REFS/@*[name() = $REFS-ATTRIBUTE-NAME],'\s+')">
            <xsl:if test="not(matches($RELEVANT-ID-LIST, concat('(^| )', ., '( |$)')))">
                <xsl:text>F</xsl:text>
            </xsl:if>
        </xsl:for-each>
      </xsl:variable>

      <xsl:choose>
        <xsl:when test="contains($CHECK-RESULT, 'F')">
            <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="true()"/>
        </xsl:otherwise>
      </xsl:choose>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:COUNT-NODE-NEGATIVE-CHARGE"
                 as="xs:decimal">

      <xsl:param name="NODE-ID" as="xs:string"/>
      <xsl:param name="DEFINED-ELECTRON-COUNT" as="xs:integer"/>
      <xsl:param name="SHARED-ELECTRON-COUNT" as="xs:integer"/>
      <xsl:param name="RELEVANT-SHARE-ELEMENTS" as="node()*"/>

      <xsl:variable name="FRACTIONS-OF-SHARED-ELECTRON-COUNTS">
        <xsl:for-each select="$RELEVANT-SHARE-ELEMENTS">
            <xsl:value-of select="xs:decimal(subsequence(tokenize(./@fractions, '\s+'),                                   index-of(tokenize(./@idrefs, '\s+'), $NODE-ID), 1))                                   * xs:decimal(./parent::ucm:particle/@counts)"/>
            <xsl:if test="position() != last()">
                <xsl:text> </xsl:text>
            </xsl:if>
        </xsl:for-each>
      </xsl:variable>

      <xsl:value-of select="$DEFINED-ELECTRON-COUNT - $SHARED-ELECTRON-COUNT + sum(                           for $i in tokenize($FRACTIONS-OF-SHARED-ELECTRON-COUNTS,' ')                           return xs:decimal($i))"/>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:COUNT-NODE-BONDING-ELECTRONS"
                 as="xs:integer">

      <xsl:param name="NODE-ELEMENT" as="node()"/>
      <xsl:param name="RELEVANT-BOND-ELEMENTS" as="node()*"/>
      <xsl:param name="RELEVANT-PARTICLE-ELEMENTS" as="node()*"/>

      <xsl:variable name="PARENT-STRUCTURE-ELEMENT"
                    select="$NODE-ELEMENT/parent::ucm:structure"/>
      <xsl:variable name="PARENT-STRUCTURE-ELEMENT-ID-REGEX"
                    select="concat('(^| )', $PARENT-STRUCTURE-ELEMENT/@id, '( |$)')"/>
      <xsl:variable name="NODE-ELEMENT-ID-REGEX"
                    select="concat('(^| )', $NODE-ELEMENT/@id, '( |$)')"/>
      <xsl:variable name="NODE-COUNT" select="count($PARENT-STRUCTURE-ELEMENT/ucm:node)"/>

      <xsl:variable name="SINGLE-BONDS-ELECTRON-COUNT"
                    select="count($RELEVANT-BOND-ELEMENTS[matches(@idrefs, $NODE-ELEMENT-ID-REGEX)                           and @order = 'S'])"/>
      <xsl:variable name="DOUBLE-BONDS-ELECTRON-COUNT"
                    select="count($RELEVANT-BOND-ELEMENTS[matches(@idrefs, $NODE-ELEMENT-ID-REGEX)                           and @order = 'D']) * 2"/>
      <xsl:variable name="TRIPLE-BONDS-ELECTRON-COUNT"
                    select="count($RELEVANT-BOND-ELEMENTS[matches(@idrefs, $NODE-ELEMENT-ID-REGEX)                           and @order = 'T']) * 3"/>
      <xsl:variable name="QUADRUPLE-BONDS-ELECTRON-COUNT"
                    select="count($RELEVANT-BOND-ELEMENTS[matches(@idrefs, $NODE-ELEMENT-ID-REGEX)                           and @order = 'Q']) * 4"/>
      <xsl:variable name="OTHER-BONDS-ELECTRON-COUNT"
                    select="sum($RELEVANT-PARTICLE-ELEMENTS[matches(@idrefs, $NODE-ELEMENT-ID-REGEX)]/@counts)                           + (sum($RELEVANT-PARTICLE-ELEMENTS[matches(@idrefs,                           $PARENT-STRUCTURE-ELEMENT-ID-REGEX)]/@counts) div $NODE-COUNT)"/>

      <xsl:value-of select="$SINGLE-BONDS-ELECTRON-COUNT + $DOUBLE-BONDS-ELECTRON-COUNT                           + $TRIPLE-BONDS-ELECTRON-COUNT + $QUADRUPLE-BONDS-ELECTRON-COUNT                           + $OTHER-BONDS-ELECTRON-COUNT"/>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:OUTPUT-ERROR-MESSAGE"
                 as="xs:string">

      <xsl:param name="NODE" as="node()"/>
      <xsl:param name="MESSAGE-LINES" as="xs:string*"/>

      <xsl:value-of select="concat('&#xA;Error in the &#34;', lf:OUTPUT-PATH-TO-NODE($NODE),                           '&#34; element:&#xA;  ', string-join($MESSAGE-LINES, '&#xA;  '))"/>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:OUTPUT-INDEX-OF-NODE"
                 as="xs:integer">
    <!--Adapted from the souce code released under the GNU LGPL 3 license at
        "http://www.xsltfunctions.com/xsl/functx_index-of-node.html".-->

      <xsl:param name="NODES" as="node()*"/>
      <xsl:param name="CURRENT-NODE" as="node()"/>

      <xsl:value-of select="for $INDEX in (1 to count($NODES))                           return $INDEX[$NODES[$INDEX] is $CURRENT-NODE]"/>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:OUTPUT-PATH-TO-NODE"
                 as="xs:string">
    <!--Adapted from the souce code released under the GNU LGPL 3 license at
        "http://www.xsltfunctions.com/xsl/functx_path-to-node-with-pos.html".-->

      <xsl:param name="NODE" as="node()"/>

      <xsl:variable name="NAMES" as="xs:string*">
        <xsl:for-each select="$NODE/ancestor-or-self::*">
            <xsl:variable name="NODE-NAME" select="name(.)"/>
            <xsl:variable name="SAME-NAME-NODES" select="./parent::*/*[name() = $NODE-NAME]"/>
            <xsl:sequence select="concat($NODE-NAME,                                   if (count($SAME-NAME-NODES) &gt; 1)                                   then concat('[', lf:OUTPUT-INDEX-OF-NODE($SAME-NAME-NODES, .), ']')                                   else '')"/>
        </xsl:for-each>
      </xsl:variable>

      <xsl:sequence select="string-join($NAMES, '/')"/>

   </xsl:function>
   <xsl:function xmlns="http://purl.oclc.org/dsdl/schematron"
                 name="lf:SORT"
                 as="item()*">

      <xsl:param name="SEQUENCE" as="item()*"/>

      <xsl:for-each select="$SEQUENCE">
        <xsl:sort select="."/>
        <xsl:copy-of select="."/>
      </xsl:for-each>

   </xsl:function>

   <!--DEFAULT RULES-->


   <!--MODE: SCHEMATRON-SELECT-FULL-PATH-->
   <!--This mode can be used to generate an ugly though full XPath for locators-->
   <xsl:template match="*" mode="schematron-select-full-path">
      <xsl:apply-templates select="." mode="schematron-get-full-path"/>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-->
   <!--This mode can be used to generate an ugly though full XPath for locators-->
   <xsl:template match="*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">
            <xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>*:</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>[namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:variable name="preceding"
                    select="count(preceding-sibling::*[local-name()=local-name(current())                                   and namespace-uri() = namespace-uri(current())])"/>
      <xsl:text>[</xsl:text>
      <xsl:value-of select="1+ $preceding"/>
      <xsl:text>]</xsl:text>
   </xsl:template>
   <xsl:template match="@*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">@<xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>@*[local-name()='</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>' and namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-2-->
   <!--This mode can be used to generate prefixed XPath for humans-->
   <xsl:template match="node() | @*" mode="schematron-get-full-path-2">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="not(self::*)">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
   </xsl:template>
   <!--MODE: SCHEMATRON-FULL-PATH-3-->
   <!--This mode can be used to generate prefixed XPath for humans 
	(Top-level element has index)-->
   <xsl:template match="node() | @*" mode="schematron-get-full-path-3">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="parent::*">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="not(self::*)">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
   </xsl:template>

   <!--MODE: GENERATE-ID-FROM-PATH -->
   <xsl:template match="/" mode="generate-id-from-path"/>
   <xsl:template match="text()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.text-', 1+count(preceding-sibling::text()), '-')"/>
   </xsl:template>
   <xsl:template match="comment()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.comment-', 1+count(preceding-sibling::comment()), '-')"/>
   </xsl:template>
   <xsl:template match="processing-instruction()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.processing-instruction-', 1+count(preceding-sibling::processing-instruction()), '-')"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.@', name())"/>
   </xsl:template>
   <xsl:template match="*" mode="generate-id-from-path" priority="-0.5">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="concat('.',name(),'-',1+count(preceding-sibling::*[name()=name(current())]),'-')"/>
   </xsl:template>

   <!--MODE: GENERATE-ID-2 -->
   <xsl:template match="/" mode="generate-id-2">U</xsl:template>
   <xsl:template match="*" mode="generate-id-2" priority="2">
      <xsl:text>U</xsl:text>
      <xsl:number level="multiple" count="*"/>
   </xsl:template>
   <xsl:template match="node()" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>n</xsl:text>
      <xsl:number count="node()"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="string-length(local-name(.))"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="translate(name(),':','.')"/>
   </xsl:template>
   <!--Strip characters-->
   <xsl:template match="text()" priority="-1"/>

   <!--SCHEMA SETUP-->
   <xsl:template match="/">
      <xsl:apply-templates select="/" mode="M4"/>
      <xsl:apply-templates select="/" mode="M5"/>
      <xsl:apply-templates select="/" mode="M6"/>
      <xsl:apply-templates select="/" mode="M7"/>
      <xsl:apply-templates select="/" mode="M8"/>
      <xsl:apply-templates select="/" mode="M9"/>
      <xsl:apply-templates select="/" mode="M10"/>
      <xsl:apply-templates select="/" mode="M11"/>
      <xsl:apply-templates select="/" mode="M12"/>
      <xsl:apply-templates select="/" mode="M13"/>
      <xsl:apply-templates select="/" mode="M14"/>
      <xsl:apply-templates select="/" mode="M15"/>
      <xsl:apply-templates select="/" mode="M16"/>
      <xsl:apply-templates select="/" mode="M17"/>
      <xsl:apply-templates select="/" mode="M18"/>
      <xsl:apply-templates select="/" mode="M19"/>
      <xsl:apply-templates select="/" mode="M20"/>
      <xsl:apply-templates select="/" mode="M21"/>
      <xsl:apply-templates select="/" mode="M22"/>
      <xsl:apply-templates select="/" mode="M23"/>
      <xsl:apply-templates select="/" mode="M24"/>
      <xsl:apply-templates select="/" mode="M25"/>
      <xsl:apply-templates select="/" mode="M26"/>
      <xsl:apply-templates select="/" mode="M27"/>
      <xsl:apply-templates select="/" mode="M28"/>
      <xsl:apply-templates select="/" mode="M29"/>
      <xsl:apply-templates select="/" mode="M30"/>
      <xsl:apply-templates select="/" mode="M31"/>
      <xsl:apply-templates select="/" mode="M32"/>
      <xsl:apply-templates select="/" mode="M33"/>
      <xsl:apply-templates select="/" mode="M34"/>
      <xsl:apply-templates select="/" mode="M35"/>
      <xsl:apply-templates select="/" mode="M36"/>
      <xsl:apply-templates select="/" mode="M37"/>
      <xsl:apply-templates select="/" mode="M38"/>
      <xsl:apply-templates select="/" mode="M39"/>
      <xsl:apply-templates select="/" mode="M40"/>
      <xsl:apply-templates select="/" mode="M41"/>
      <xsl:apply-templates select="/" mode="M42"/>
      <xsl:apply-templates select="/" mode="M43"/>
   </xsl:template>

   <!--SCHEMATRON PATTERNS-->


   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="/*" priority="1000" mode="M4">

		<!--REPORT -->
      <xsl:if test="true()">
        <xsl:value-of select="string('&#xA;')"/>
      </xsl:if>
      <xsl:apply-templates select="*" mode="M4"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M4"/>
   <xsl:template match="@*|node()" priority="-2" mode="M4">
      <xsl:apply-templates select="*" mode="M4"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:structure[@charge]" priority="1002" mode="M5">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="sum(./*/@charge) = ./@charge"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The sum of charges of the node or structure child elements is not equal',                               'to the charge attribute value on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M5"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:structure and @idrefs]"
                 priority="1001"
                 mode="M5">
      <xsl:variable name="ID" select="./@id"/>
      <xsl:variable name="IDREFS" select="./@idrefs"/>
      <xsl:variable name="NODE-DEFINITION"
                    select="//ucm:node[parent::ucm:define and @id = $IDREFS]"/>
      <xsl:variable name="DEFINED-ELECTRON-COUNT"
                    select="xs:integer(sum($NODE-DEFINITION/ucm:particle[contains(@type, 'E')]/@counts))"/>
      <xsl:variable name="SHARED-ELECTRON-COUNT"
                    select="xs:integer(sum(//ucm:particle[ucm:share and @idrefs = $ID]/@counts))"/>
      <xsl:variable name="RELEVANT-SHARE-ELEMENTS"
                    select="//ucm:share[contains(@idrefs, $ID)]"/>
      <xsl:variable name="NEGATIVE-CHARGE"
                    select="lf:COUNT-NODE-NEGATIVE-CHARGE($ID, $DEFINED-ELECTRON-COUNT, $SHARED-ELECTRON-COUNT,                     $RELEVANT-SHARE-ELEMENTS)"/>
      <xsl:variable name="POSITIVE-CHARGE"
                    select="sum($NODE-DEFINITION/ucm:particle[@type = 'P']/@counts)"/>
      <xsl:variable name="TOTAL-CHARGE" select="$POSITIVE-CHARGE - $NEGATIVE-CHARGE"/>
      <xsl:variable name="CURRENT-CHARGE" select="if (./@charge) then ./@charge else 0"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$TOTAL-CHARGE = $CURRENT-CHARGE"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The charge attribute value on this element must be ',                               $TOTAL-CHARGE, ', because it contains'),                               concat($POSITIVE-CHARGE, ' proton(s) and ', $NEGATIVE-CHARGE,                               ' electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M5"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:structure and not(@idrefs)]"
                 priority="1000"
                 mode="M5">
      <xsl:variable name="ID" select="./@id"/>
      <xsl:variable name="DEFINED-ELECTRON-COUNT"
                    select="xs:integer(sum(./ucm:particle[contains(@type, 'E')]/@counts))"/>
      <xsl:variable name="SHARED-ELECTRON-COUNT"
                    select="xs:integer(sum(//ucm:particle[ucm:share and @idrefs = $ID]/@counts))"/>
      <xsl:variable name="RELEVANT-SHARE-ELEMENTS"
                    select="//ucm:share[contains(@idrefs, $ID)]"/>
      <xsl:variable name="NEGATIVE-CHARGE"
                    select="lf:COUNT-NODE-NEGATIVE-CHARGE($ID, $DEFINED-ELECTRON-COUNT, $SHARED-ELECTRON-COUNT,                     $RELEVANT-SHARE-ELEMENTS)"/>
      <xsl:variable name="POSITIVE-CHARGE"
                    select="sum(./ucm:particle[@type = 'P']/@counts)"/>
      <xsl:variable name="TOTAL-CHARGE" select="$POSITIVE-CHARGE - $NEGATIVE-CHARGE"/>
      <xsl:variable name="CURRENT-CHARGE" select="if (./@charge) then ./@charge else 0"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$TOTAL-CHARGE = $CURRENT-CHARGE"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The charge attribute value on this element must be ',                               $TOTAL-CHARGE, ', because it contains'),                               concat($POSITIVE-CHARGE, ' proton(s) and ', $NEGATIVE-CHARGE,                               ' electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M5"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M5"/>
   <xsl:template match="@*|node()" priority="-2" mode="M5">
      <xsl:apply-templates select="*" mode="M5"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[@fractions]" priority="1000" mode="M6">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="sum(for $i in tokenize(./@fractions,'\s+') return xs:decimal($i)) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The sum of all numbers in the fractions attribute on this element is not',                               'equal to one.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M6"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M6"/>
   <xsl:template match="@*|node()" priority="-2" mode="M6">
      <xsl:apply-templates select="*" mode="M6"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[matches(@format, '^(UNITSML|BIBTEXML)$')]"
                 priority="1001"
                 mode="M7">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'define'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The format attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on define elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M7"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[@format and not(matches(@format, '^(UCM|UNITSML|BIBTEXML)$'))]"
                 priority="1000"
                 mode="M7">
      <xsl:variable name="INCHI-FORMULA-REGEX"
                    select="string('(/([0-9]*[A-IK-PR-Z][a-ik-pr-vy]?[0-9]*\.?)+)?')"/>
      <xsl:variable name="SMILES-CHARS-REGEX"
                    select="string('0-9A-IK-PR-Za-ik-pr-vy*@:.=#$\\/%()\[\]+\-')"/>
      <xsl:variable name="ENABLED-CONTENT-REGEX"
                    select="if (@format = 'IUPAC-PREFERRED-NAME-U'                     or @format = 'IUPAC-GENERAL-NAME'                     or @format = 'CA-INDEX-NAME') then '^\s*(\S| )+\s*$'                     else (if (@format = 'CAS-RN-U') then '^\s*[0-9]{1,7}-[0-9]{2}-[0-9]\s*$'                     else (if (@format = 'REAXYS-RN-U'                     or @format = 'CHEMSPIDER-ID-U'                     or @format = 'PUBCHEM-CID-U'                     or @format = 'PUBCHEM-SID') then '^\s*[0-9]+\s*$'                     else (if (@format = 'INCHI')                     then concat('^\s*1', $INCHI-FORMULA-REGEX,                     '(/[bcfhimopqrst][0-9A-IK-PR-Za-ik-pr-vy.,;()?*+\-]*)*\s*$')                     else (if (@format = 'INCHI-KEY') then '^\s*[A-Z]{14}-[A-Z]{8}NA-[A-Z]\s*$'                     else (if (@format = 'S-INCHI-U')                     then concat('^\s*1S', $INCHI-FORMULA-REGEX,                     '(/[bchimpqst][0-9DHThm.,;()?*+\-]*)*\s*$')                     else (if (@format = 'S-INCHI-KEY') then '^\s*[A-Z]{14}-[A-Z]{8}SA-[A-Z]\s*$'                     else (if (@format = 'SMILES')                     then concat('^\s*[', $SMILES-CHARS-REGEX, ']+\s*$')                     else (if (@format = 'SMARTS')                     then concat('^\s*[', $SMILES-CHARS-REGEX, 'x;,~?!&amp;]+\s*$')                     else (if (@format = 'SLN')                     then '^\s*[0-9A-Za-z*@;:.,~=#(){}\[\]+\-\\|!&amp;&#34;&lt;&gt;]+\s*$'                     else '^$')))))))))"/>
      <xsl:variable name="NOT-ENABLED-CONTENT-REGEX"
                    select="if (@format = 'SLN') then '^\s*.*-&gt;.*\s*$' else '^$'"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'structure'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The format attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on structure elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="matches(./text(), $ENABLED-CONTENT-REGEX)                       and not(matches(./text(), $NOT-ENABLED-CONTENT-REGEX))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The content inside this element does not match the regular expresion',                               concat('pattern for the given format attribute value &#34;', ./@format,                               '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M7"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M7"/>
   <xsl:template match="@*|node()" priority="-2" mode="M7">
      <xsl:apply-templates select="*" mode="M7"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[@format = 'UCM']" priority="1002" mode="M8">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./*) = count(./ucm:*)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('On this element the format attribute has the value &#34;UCM&#34;, but',                               'not all child elements inside the element are from UCM namespace.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M8"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[@format = 'UNITSML']" priority="1001" mode="M8">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./*) = count(./unitsml:*)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('On this element the format attribute has the value &#34;UNITSML&#34;, but',                               'not all child elements inside the element are from UnitsML namespace.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M8"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[@format = 'BIBTEXML']" priority="1000" mode="M8">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./*) = count(./bibtexml:*)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('On this element the format attribute has the value &#34;BIBTEXML&#34;, but',                               'not all child elements inside the element are from BibTeXML namespace.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M8"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M8"/>
   <xsl:template match="@*|node()" priority="-2" mode="M8">
      <xsl:apply-templates select="*" mode="M8"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[@type = 'SBST']" priority="1002" mode="M9">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@format = 'UCM'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The type attribute on this element has the value &#34;SBST&#34;, thus',                               'the value of the format attribute must be &#34;UCM&#34;, but instead',                               concat('it is &#34;', ./@format, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M9"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[@type = 'STQR']" priority="1001" mode="M9">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@format = 'UCM')"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The type attribute on this element has the value &#34;STQR&#34;, thus',                               'the value of the format attribute cannot be &#34;UCM&#34;.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M9"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[@type = 'STID']" priority="1000" mode="M9">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(matches(@format, '^(UCM|SMILES|SMARTS|SLN)$'))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The type attribute on this element has the value &#34;STID&#34;, thus',                               concat('the value of the format attribute cannot be &#34;',                               ./@format, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M9"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M9"/>
   <xsl:template match="@*|node()" priority="-2" mode="M9">
      <xsl:apply-templates select="*" mode="M9"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[@idrefs]" priority="1000" mode="M10">
      <xsl:variable name="IDREFS" select="tokenize(./@idrefs,'\s+')"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="deep-equal(lf:SORT(distinct-values($IDREFS)), lf:SORT($IDREFS))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference is repeated more than once in the idrefs',                               'attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M10"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M10"/>
   <xsl:template match="@*|node()" priority="-2" mode="M10">
      <xsl:apply-templates select="*" mode="M10"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:*[matches(@type, '^(P|N|E|N?BE)$')]"
                 priority="1003"
                 mode="M11">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'particle'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on particle elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M11"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[matches(@type, '^(PR|CN|ER)$')]"
                 priority="1002"
                 mode="M11">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'property'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on property elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M11"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[matches(@type, '^(ST|STQR)$')]"
                 priority="1001"
                 mode="M11">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'structure' and parent::ucm:*[local-name() = 'ucm']"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on structure elements that are the childs of an ucm',                               'element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M11"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:*[matches(@type, '^(SBST|STID)$')]"
                 priority="1000"
                 mode="M11">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="local-name() = 'structure' and parent::ucm:*[local-name() = 'structure']"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               'which is enabled only on structure elements that are the childs of another',                               'structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M11"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M11"/>
   <xsl:template match="@*|node()" priority="-2" mode="M11">
      <xsl:apply-templates select="*" mode="M11"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[ucm:particle]" priority="1001" mode="M12">
      <xsl:variable name="ID-REGEX" select="concat('(^| )', ./@id, '( |$)')"/>
      <xsl:variable name="BOND-BONDING-ELECTRONS"
                    select="sum(./ucm:particle[@type = 'BE']/@counts)"/>
      <xsl:variable name="REQUIRED-BOND-BONDING-ELECTRONS"
                    select="sum(//ucm:particle[matches(@idrefs, $ID-REGEX) and @type = 'BE']/@counts)"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$BOND-BONDING-ELECTRONS &gt;= $REQUIRED-BOND-BONDING-ELECTRONS"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('This element contains ', $BOND-BONDING-ELECTRONS,                               ' bonding electron(s), but all bond elements that refer'),                               concat('to this element require ', $REQUIRED-BOND-BONDING-ELECTRONS,                               ' bonding electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M12"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:bond[not(ucm:particle)]" priority="1000" mode="M12">
      <xsl:variable name="ID-REGEX" select="concat('(^| )', ./@id, '( |$)')"/>
      <xsl:variable name="BOND-BONDING-ELECTRONS"
                    select="if (@order = 'S') then 2                                                   else (if (@order = 'D') then 4                                                   else (if (@order = 'T') then 6                                                   else (if (@order = 'Q') then 8                                                   else 0)))"/>
      <xsl:variable name="REQUIRED-BOND-BONDING-ELECTRONS"
                    select="sum(//ucm:particle[matches(@idrefs, $ID-REGEX) and @type = 'BE']/@counts)"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$BOND-BONDING-ELECTRONS &gt;= $REQUIRED-BOND-BONDING-ELECTRONS"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('This element contains ', $BOND-BONDING-ELECTRONS,                               ' bonding electron(s), but all bond elements that refer'),                               concat('to this element require ', $REQUIRED-BOND-BONDING-ELECTRONS,                               ' bonding electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M12"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M12"/>
   <xsl:template match="@*|node()" priority="-2" mode="M12">
      <xsl:apply-templates select="*" mode="M12"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[not(@idrefs)]" priority="1001" mode="M13">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./ucm:join) &gt;= 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which does not have an idrefs attribute, is missing one or more',                               'mandatory join child element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M13"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:bond[@idrefs]" priority="1000" mode="M13">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./ucm:join)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which has the idrefs attribute, cannot contain any join child',                               'elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M13"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M13"/>
   <xsl:template match="@*|node()" priority="-2" mode="M13">
      <xsl:apply-templates select="*" mode="M13"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[matches(@order, '^(P?S|P?D|P?T|P?Q)$')]"
                 priority="1000"
                 mode="M14">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@idrefs"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The order attribute on this element has the value &#34;',                               ./@order, '&#34;,'),                               'thus the context with the idrefs attribute is mandatory, but the idrefs',                               'attribute is missing on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M14"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M14"/>
   <xsl:template match="@*|node()" priority="-2" mode="M14">
      <xsl:apply-templates select="*" mode="M14"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[@idrefs]" priority="1000" mode="M15">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = 2"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There are not two id references in the idrefs attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //(ucm:node | ucm:point)[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node or point element inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M15"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M15"/>
   <xsl:template match="@*|node()" priority="-2" mode="M15">
      <xsl:apply-templates select="*" mode="M15"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[not(ucm:particle) and @idrefs]"
                 priority="1000"
                 mode="M16">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:node[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element does not contain any particle child element, but at least one',                               'id reference in its idrefs attribute does not refer to the node element',                               'inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M16"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M16"/>
   <xsl:template match="@*|node()" priority="-2" mode="M16">
      <xsl:apply-templates select="*" mode="M16"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[matches(@order, '^(PS|PD|PT|PQ|A|DL)$')]"
                 priority="1001"
                 mode="M17">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./ucm:particle[@type = 'BE']) &gt;= 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The order attribute on this element has the value &#34;',                               ./@order, '&#34;,'),                               'but electrons participating in the bond are not specified using one or more',                               'particle elements inside this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M17"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:bond[matches(@order, '^(I|H|DIP)$')]"
                 priority="1000"
                 mode="M17">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./ucm:particle) = 0"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The order attribute on this element has the value &#34;',                               ./@order, '&#34;,'),                               'thus particle child elements for this element are not enabled, but there',                               'is at least one.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M17"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M17"/>
   <xsl:template match="@*|node()" priority="-2" mode="M17">
      <xsl:apply-templates select="*" mode="M17"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:bond[ucm:particle and matches(@order, '^(P?S|P?D|P?T|P?Q)$')]"
                 priority="1000"
                 mode="M18">
      <xsl:variable name="BOND-BONDING-ELECTRONS"
                    select="sum(./ucm:particle[@type = 'BE']/@counts)"/>
      <xsl:variable name="REQUIRED-BOND-BONDING-ELECTRONS"
                    select="if (@order = 'PS') then 1                                                   else (if (@order = 'S') then 2                                                   else (if (@order = 'PD') then 3                                                   else (if (@order = 'D') then 4                                                   else (if (@order = 'PT') then 5                                                   else (if (@order = 'T') then 6                                                   else (if (@order = 'PQ') then 7                                                   else (if (@order = 'Q') then 8                                                   else 0)))))))"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$BOND-BONDING-ELECTRONS = $REQUIRED-BOND-BONDING-ELECTRONS"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('This element contains ', $BOND-BONDING-ELECTRONS,                               ' bonding electron(s), but the order attribute on this'),                               concat('element has the value &#34;', ./@order, '&#34;, which requires ',                               $REQUIRED-BOND-BONDING-ELECTRONS, ' bonding electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M18"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M18"/>
   <xsl:template match="@*|node()" priority="-2" mode="M18">
      <xsl:apply-templates select="*" mode="M18"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:description[parent::ucm:define]"
                 priority="1000"
                 mode="M19">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@id"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is no id attribute on this element, but on the description element',                               'that is inside a define element the id attribute is mandatory.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M19"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M19"/>
   <xsl:template match="@*|node()" priority="-2" mode="M19">
      <xsl:apply-templates select="*" mode="M19"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:description[@idrefs]" priority="1000" mode="M20">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:description[parent::ucm:define and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the description element inside a define element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M20"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M20"/>
   <xsl:template match="@*|node()" priority="-2" mode="M20">
      <xsl:apply-templates select="*" mode="M20"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:description[@litrefs]" priority="1000" mode="M21">
      <xsl:variable name="LITREFS" select="tokenize(./@litrefs,'\s+')"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="deep-equal(lf:SORT(distinct-values($LITREFS)), lf:SORT($LITREFS))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference is repeated more than once in the litrefs',                               'attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('litrefs', ., //bibtexml:entry[@id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the litrefs attribute on this element does not',                               'refer to the literature reference defined using BibTeXML inside a define',                               'element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M21"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M21"/>
   <xsl:template match="@*|node()" priority="-2" mode="M21">
      <xsl:apply-templates select="*" mode="M21"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:join[@idrefs]" priority="1000" mode="M22">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) &gt;= 2"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is less than two id references in the idrefs attribute on this',                               'element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //(ucm:node | ucm:point)[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node or point element inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M22"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M22"/>
   <xsl:template match="@*|node()" priority="-2" mode="M22">
      <xsl:apply-templates select="*" mode="M22"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:join[parent::ucm:bond and not(./parent::ucm:bond/ucm:particle)]"
                 priority="1000"
                 mode="M23">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:node[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The parent bond element of this element does not contain any particle child',                               'element, but at least one id reference in the idrefs attribute on this element',                               'does not refer to the node element inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M23"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M23"/>
   <xsl:template match="@*|node()" priority="-2" mode="M23">
      <xsl:apply-templates select="*" mode="M23"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:node[not(@idrefs)]" priority="1001" mode="M24">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./ucm:particle) &gt;= 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which does not have an idrefs attribute, is missing one or more',                               'mandatory particle child element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M24"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:strucure and @idrefs]"
                 priority="1000"
                 mode="M24">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./ucm:particle)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which is inside a strucure element and has the idrefs',                               'attribute, cannot contain any particle child elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M24"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M24"/>
   <xsl:template match="@*|node()" priority="-2" mode="M24">
      <xsl:apply-templates select="*" mode="M24"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:structure and @idrefs]"
                 priority="1001"
                 mode="M25">
      <xsl:variable name="IDREFS" select="./@idrefs"/>
      <xsl:variable name="NODE-BONDING-ELECTRONS"
                    select="sum(//ucm:node[parent::ucm:define and @id = $IDREFS]/ucm:particle[@type = 'BE']/@counts)"/>
      <xsl:variable name="REQUIRED-NODE-BONDING-ELECTRONS"
                    select="lf:COUNT-NODE-BONDING-ELECTRONS(.,                     //ucm:bond[not(ucm:particle)],                     //ucm:particle[@type = 'BE'])"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$NODE-BONDING-ELECTRONS = $REQUIRED-NODE-BONDING-ELECTRONS"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The node definition of this element contains ',                               $NODE-BONDING-ELECTRONS, ' bonding electron(s), but all'),                               concat('bonds of this element require ',                               $REQUIRED-NODE-BONDING-ELECTRONS, ' bonding electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M25"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:structure and not(@idrefs)]"
                 priority="1000"
                 mode="M25">
      <xsl:variable name="NODE-BONDING-ELECTRONS"
                    select="sum(./ucm:particle[@type = 'BE']/@counts)"/>
      <xsl:variable name="REQUIRED-NODE-BONDING-ELECTRONS"
                    select="lf:COUNT-NODE-BONDING-ELECTRONS(.,                     //ucm:bond[not(ucm:particle)],                     //ucm:particle[@type = 'BE'])"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$NODE-BONDING-ELECTRONS = $REQUIRED-NODE-BONDING-ELECTRONS"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('This element contains ', $NODE-BONDING-ELECTRONS,                               ' bonding electron(s), but all bonds of this element'),                               concat('require ', $REQUIRED-NODE-BONDING-ELECTRONS,                               ' bonding electron(s).')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M25"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M25"/>
   <xsl:template match="@*|node()" priority="-2" mode="M25">
      <xsl:apply-templates select="*" mode="M25"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:define]" priority="1000" mode="M26">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@charge)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is a charge attribute on this element, but on the node element that',                               'is inside a define element the charge attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@x) or not(./@y) or not(./@z)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element contains coordinates in 3-dimensional space (at least one of',                               'the x, y or z attributes), but on the node element that is inside a define',                               'element the attributes for coordinates must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./ucm:stereo)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element contains a stereo child element, but on the node element that',                               'is inside a define element the stereo child element must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M26"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M26"/>
   <xsl:template match="@*|node()" priority="-2" mode="M26">
      <xsl:apply-templates select="*" mode="M26"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:node[parent::ucm:define]" priority="1001" mode="M27">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@idrefs)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is an idrefs attribute on this element, but on the node element that',                               'is inside a define element the idrefs attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M27"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:node[@idrefs]" priority="1000" mode="M27">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is not one id reference in the idrefs attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:node[parent::ucm:define and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node element inside a define element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M27"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M27"/>
   <xsl:template match="@*|node()" priority="-2" mode="M27">
      <xsl:apply-templates select="*" mode="M27"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:bond]" priority="1001" mode="M28">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@idrefs"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is no idrefs attribute on this element, but on the particle element',                               'that is inside a bond element the idrefs attribute is mandatory.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M28"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:node]" priority="1000" mode="M28">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@idrefs)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is an idrefs attribute on this element, but on the particle element',                               'that is inside a node element the idrefs attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./ucm:share)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which is inside a node element, cannot contain any share child',                               'elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M28"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M28"/>
   <xsl:template match="@*|node()" priority="-2" mode="M28">
      <xsl:apply-templates select="*" mode="M28"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[count(tokenize(./@counts,'\s+')) &gt; 1]"
                 priority="1001"
                 mode="M29">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@type = 'N'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The counts attribute on this element has more than one number in its value,',                               'thus the value of the type attribute on this element must be &#34;N&#34;,',                               concat('but instead it is &#34;', ./@type, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@counts,'\s+')) = count(tokenize(./@fractions,'\s+'))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('On this element the occurrence ratio of each count in the counts attribute',                               'is not specified properly using the fractions attribute.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M29"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:particle[count(tokenize(./@counts,'\s+')) = 1]"
                 priority="1000"
                 mode="M29">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@fractions)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is a fractions attribute on this element, but on the particle element,',                               'with one number in its counts attribute the fractions attribute must be',                               'omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M29"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M29"/>
   <xsl:template match="@*|node()" priority="-2" mode="M29">
      <xsl:apply-templates select="*" mode="M29"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[@idrefs]" priority="1000" mode="M30">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is not one id reference in the idrefs attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', .,                       (//ucm:bond[@id] | //ucm:structure[@id] | //ucm:node[parent::ucm:structure and @id]))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the bond or structure element or to the node element inside',                               'the structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M30"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M30"/>
   <xsl:template match="@*|node()" priority="-2" mode="M30">
      <xsl:apply-templates select="*" mode="M30"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:bond and @type]"
                 priority="1001"
                 mode="M31">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@type = 'BE'"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;, but'),                               'only the &#34;BE&#34; value of the type attribute is enabled on the particle',                               'element that is inside the bond element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M31"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:node and @type]"
                 priority="1000"
                 mode="M31">
      <xsl:variable name="TYPE" select="./@type"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./parent::ucm:node/ucm:particle[@type = $TYPE]) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(./parent::ucm:node,                               ('This element cannot contain more than one particle element with the type',                               concat('attribute that has the value &#34;', ./@type, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M31"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M31"/>
   <xsl:template match="@*|node()" priority="-2" mode="M31">
      <xsl:apply-templates select="*" mode="M31"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:node and @type = 'NBE']"
                 priority="1000"
                 mode="M32">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./parent::ucm:node/ucm:particle[@type = 'BE']) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(./parent::ucm:node,                               ('This element must contain the particle element with the type attribute that',                               'has the value &#34;BE&#34;, because it contains the particle element that',                               concat('has the type attribute with the value &#34;', ./@type, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M32"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M32"/>
   <xsl:template match="@*|node()" priority="-2" mode="M32">
      <xsl:apply-templates select="*" mode="M32"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:particle[parent::ucm:node and (@type = 'BE' or @type = 'NBE')]"
                 priority="1000"
                 mode="M33">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(./parent::ucm:node/ucm:particle[@type = 'E']) = 0"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(./parent::ucm:node,                               ('This element cannot contain the particle element with the type attribute that',                               'has the value &#34;E&#34;, because it contains the particle element that has',                               concat('the type attribute with the value &#34;', ./@type, '&#34;.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M33"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M33"/>
   <xsl:template match="@*|node()" priority="-2" mode="M33">
      <xsl:apply-templates select="*" mode="M33"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[not(@idrefs)]" priority="1001" mode="M34">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./ucm:values"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which does not have an idrefs attribute, is missing one',                               'mandatory values child element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M34"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@idrefs]" priority="1000" mode="M34">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./ucm:property) and not(./ucm:values)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('This element, which has the idrefs attribute, cannot contain any',                               'property or values child elements.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M34"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M34"/>
   <xsl:template match="@*|node()" priority="-2" mode="M34">
      <xsl:apply-templates select="*" mode="M34"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[parent::ucm:define]"
                 priority="1001"
                 mode="M35">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@idrefs)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is an idrefs attribute on this element, but on the property element',                               'that is inside a define element the idrefs attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M35"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@idrefs]" priority="1000" mode="M35">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is not one id reference in the idrefs attribute on this element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:property[parent::ucm:define and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the property element inside a define element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M35"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M35"/>
   <xsl:template match="@*|node()" priority="-2" mode="M35">
      <xsl:apply-templates select="*" mode="M35"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[@quantity]" priority="1000" mode="M36">
      <xsl:variable name="QUANTITY" select="substring-after(./@quantity, '#')"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(//unitsml:Quantity[@xml:id = $QUANTITY]) = 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The quantity attribute value on this element does not refer to the unique',                               'xml:id of a quantity defined using UnitsML inside a define element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M36"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M36"/>
   <xsl:template match="@*|node()" priority="-2" mode="M36">
      <xsl:apply-templates select="*" mode="M36"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[not(@idrefs)]" priority="1001" mode="M37">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@quantity"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is no quantity attribute on this element, but on the property element',                               'without the idrefs attribute the quantity attribute is mandatory.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M37"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@idrefs]" priority="1000" mode="M37">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@quantity)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is a quantity attribute on this element, but on the property element',                               'that has the idrefs attribute the quantity attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M37"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M37"/>
   <xsl:template match="@*|node()" priority="-2" mode="M37">
      <xsl:apply-templates select="*" mode="M37"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[@type = 'PR']" priority="1002" mode="M38">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="parent::ucm:*[local-name() = 'bond' or local-name() = 'node'                       or local-name() = 'particle' or local-name() = 'point'                       or local-name() = 'structure' or local-name() = 'define']"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               concat('thus this element must not be a child of the ',                               name(parent::*), ' element.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M38"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@type = 'CN']" priority="1001" mode="M38">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="parent::ucm:*[local-name() = 'property' or local-name() = 'define']"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               concat('thus this element must not be a child of the ',                               name(parent::*), ' element.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M38"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@type = 'ER']" priority="1000" mode="M38">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="parent::ucm:*[local-name() = 'property']"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               (concat('The type attribute on this element has the value &#34;',                               ./@type, '&#34;,'),                               concat('thus this element must not be a child of the ',                               name(parent::*), ' element.')))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M38"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M38"/>
   <xsl:template match="@*|node()" priority="-2" mode="M38">
      <xsl:apply-templates select="*" mode="M38"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:property[not(@idrefs)]" priority="1001" mode="M39">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="./@type"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is no type attribute on this element, but on the property element',                               'without the idrefs attribute, the type attribute is mandatory.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M39"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:property[@idrefs]" priority="1000" mode="M39">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="not(./@type)"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There is a type attribute on this element, but on the property element that',                               'has the idrefs attribute the type attribute must be omitted.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M39"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M39"/>
   <xsl:template match="@*|node()" priority="-2" mode="M39">
      <xsl:apply-templates select="*" mode="M39"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:share[@idrefs]" priority="1000" mode="M40">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) &gt; 1"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There are not two or more id references in the idrefs attribute on this',                               'element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //ucm:node[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node element inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = count(tokenize(./@fractions,'\s+'))"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('On this element the sharing ratio of each id reference in the idrefs',                               'attribute is not specified properly using the fractions attribute.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M40"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M40"/>
   <xsl:template match="@*|node()" priority="-2" mode="M40">
      <xsl:apply-templates select="*" mode="M40"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:stereo[@idrefs]" priority="1000" mode="M41">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., //(ucm:node | ucm:point)[parent::ucm:structure and @id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node or point element inside a structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M41"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M41"/>
   <xsl:template match="@*|node()" priority="-2" mode="M41">
      <xsl:apply-templates select="*" mode="M41"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:stereo[parent::ucm:bond and @idrefs]"
                 priority="1002"
                 mode="M42">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="count(tokenize(./@idrefs,'\s+')) = 4"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There are not four id references in the idrefs attribute on this element',                               'although it is inside the bond element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M42"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:stereo[parent::ucm:node and @idrefs]"
                 priority="1001"
                 mode="M42">
      <xsl:variable name="IDREFS-COUNT" select="count(tokenize(./@idrefs,'\s+'))"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$IDREFS-COUNT = 4 or $IDREFS-COUNT = 5 or $IDREFS-COUNT = 7"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There are not four, five or seven id references in the idrefs attribute',                               'on this element although it is inside the node element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M42"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:stereo[parent::ucm:structure and @idrefs]"
                 priority="1000"
                 mode="M42">
      <xsl:variable name="IDREFS-COUNT" select="count(tokenize(./@idrefs,'\s+'))"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$IDREFS-COUNT = 5 or $IDREFS-COUNT = 6 or $IDREFS-COUNT = 7"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('There are not five, six or seven id references in the idrefs attribute',                               'on this element although it is inside the structure element.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M42"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M42"/>
   <xsl:template match="@*|node()" priority="-2" mode="M42">
      <xsl:apply-templates select="*" mode="M42"/>
   </xsl:template>

   <!--PATTERN -->


	  <!--RULE -->
   <xsl:template match="//ucm:stereo[parent::ucm:node and @idrefs and count(tokenize(@idrefs,'\s+')) &gt; 4]"
                 priority="1001"
                 mode="M43">
      <xsl:variable name="IDREFS" select="tokenize(./@idrefs,'\s+')"/>

		    <!--ASSERT -->
      <xsl:choose>
         <xsl:when test="$IDREFS[1] = ./parent::ucm:node/@id"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('The first id reference in the idrefs attribute on this element does not refer',                               'to the parent node element although this element is inside the node element',                               'and has more than four id references in the idrefs attribute.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M43"/>
   </xsl:template>

	  <!--RULE -->
   <xsl:template match="//ucm:stereo[parent::ucm:structure and @idrefs and count(tokenize(@idrefs,'\s+')) &gt; 5]"
                 priority="1000"
                 mode="M43">

		<!--ASSERT -->
      <xsl:choose>
         <xsl:when test="lf:CHECK-REFS('idrefs', ., ./parent::ucm:structure/descendant::ucm:node[@id])"/>
         <xsl:otherwise>
            <xsl:text/>
            <xsl:value-of select="lf:OUTPUT-ERROR-MESSAGE(.,                               ('At least one id reference in the idrefs attribute on this element does not',                               'refer to the node element that is a descendant of the parent structure element',                               'although this element is inside the structure element and has more than five',                               'id references in the idrefs attribute.'))"/>
            <xsl:text/>
            <xsl:value-of select="string('&#xA;')"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M43"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M43"/>
   <xsl:template match="@*|node()" priority="-2" mode="M43">
      <xsl:apply-templates select="*" mode="M43"/>
   </xsl:template>
</xsl:stylesheet>
